/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#ifndef PRECISEPOLYGON_H_
#define PRECISEPOLYGON_H_

#include "../../Type.h"
#include "Point2D.h"
#include "../../units/UnitConverter.h"

#include <vector>

namespace airobots {

class PrecisePolygon : public Type {

public:

	PrecisePolygon();
	PrecisePolygon(const PrecisePolygon &poly);
	virtual ~PrecisePolygon();

	static std::string getBaseTag();
	static std::string getTemplate();

	void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, PrecisePolygon &pol);
	void sanitizeLocation(xmltio::Location &loc) const;

	virtual bool operator==(const PrecisePolygon &other) const;

	void addPoint(double x, double y, LengthUnit unit);
	void addPoint(Point2D point);

	Point2D getCentroid();
	std::vector<Point2D> getPoints() const;

//	double getMinX(/*LengthUnit unit*/);
//	double getMaxX(/*LengthUnit unit*/);
//	double getMinY(/*LengthUnit unit*/);
//	double getMaxY(/*LengthUnit unit*/);
//	double getArea(/*LengthUnit unit*/);
//
//	void move(double dx, double dy/*, LengthUnit unit*/);
//	void scale(double factor);
//	void rotate(Point2D center, double angle/*, AngleUnit unit*/);
//	void rotate(double x, double y/*, LengthUnit lengthunit*/, double angle/*, AngleUnit angleUnit*/);
//
//	bool contains(double x, double y/*, LengthUnit unit*/);
//	bool contains(Point2D p);

	// generatePath, updateFromPath

protected:

	std::vector<Point2D> list;

};

}

#endif /* PRECISEPOLYGON_H_ */
