/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#include <exception>
#include <stdexcept>
#include <string>

#include "GlobalPlannerStateChange.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

std::string GlobalPlannerStateChange::getBaseTag() {
	return "GLOBALPLANNERSTATECHANGE";
}

std::string GlobalPlannerStateChange::getTemplate() {
	return "<" + getBaseTag() + "><GENERATOR/>" + Timestamp::getTemplate()
			+ "<STATE/></" + getBaseTag() + ">";
}

GlobalPlannerStateChange::GlobalPlannerStateChange() {
}

GlobalPlannerStateChange::GlobalPlannerStateChange(std::string generator,
		GlobalPlannerStateChange::State newState) :
	Type(generator), newState(newState) {

}

GlobalPlannerStateChange::State GlobalPlannerStateChange::getNewState() const {
	return newState;
}

void GlobalPlannerStateChange::setNewState(
		GlobalPlannerStateChange::State newState) {
	this->newState = newState;
}

void GlobalPlannerStateChange::sanitizeLocation(xmltio::Location &loc) const {

}

void GlobalPlannerStateChange::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	// Convert the state to a string
	string stateString;
	switch (getNewState()) {
	case GlobalPlannerStateChange::PLANNING:
		stateString = "PLANNING";
		break;
	case GlobalPlannerStateChange::FOLLOWING_PLAN:
		stateString = "FOLLOWING_PLAN";
		break;
	case GlobalPlannerStateChange::PATH_BLOCKED:
		stateString = "PATH_BLOCKED";
		break;
	default:
		runtime_error e("Illegal enum value. Fix convert(PlannerState)?");
		throw e;
		break;
	}

	// Set the value of the <PlannerStateChange> element to that string
	loc[XPath("STATE")] = stateString;

}

void GlobalPlannerStateChange::fromLocation(const xmltio::Location &loc_, GlobalPlannerStateChange &change) {

	Type::fromLocation(loc_, change);

	string stateString = extract<string> (loc_[XPath(
			"STATE")]);
	GlobalPlannerStateChange::State state;
	if (stateString == "PLANNING") {
		state = GlobalPlannerStateChange::PLANNING;
	} else if (stateString == "FOLLOWING_PLAN") {
		state = GlobalPlannerStateChange::FOLLOWING_PLAN;
	} else if (stateString == "PATH_BLOCKED") {
		state = GlobalPlannerStateChange::PATH_BLOCKED;
	} else {
		throw ParsingException("Illegal enum value for State",
				loc_.getDocumentText(true));
	}
	change.setNewState(state);

}
