/*
 * This file is part of Bielefeld Type Library (BTL).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/btl
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
 * 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
 * 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */

/*
 * Face.cpp
 *
 *  Created on: Aug 18, 2009
 *      Author: jwienke
 */

#include "Face.h"
#include "PerceptList.h"
#include <cmath>

using namespace std;
using namespace xmltio;
using namespace airobots;

const float TOLERANCE = 0.0001;

namespace airobots {

Face::Face() :
		angle(0), angleVariance(0), height(0), heightVariance(0), distance(0), distanceVariance(
				0), reliability(0), faceWidthAngle(0), lineOfSight(0), imagePositionX(
				0), imagePositionY(0), imageWidth(0), imageHeight(0) {

}

Face::~Face() {
}

std::string Face::getBaseTag() {
	return "FACE";
}

std::string Face::getTemplate() {
	return string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate()
			+ "<POSITION angle=\"\" angleVariance=\"\" height=\"\" heightVariance=\"\" distance=\"\" distanceVariance=\"\" />"
			+ "<IMAGEPOSITION x=\"\" y=\"\" width=\"\" height=\"\" />"
			+ "<DATA lineOfSight=\"\" faceWidthAngle=\"\" reliability=\"\"/>"
			+ "</" + getBaseTag() + ">";
}

std::string Face::getPerceptName() {
	return "face";
}

float Face::getAngle(AngleUnit aU) const {
	return UnitConverter::convert(this->angle, iAU, aU);
}

void Face::setAngle(float angle, AngleUnit aU) {
	this->angle = UnitConverter::convert(angle, aU, iAU);
}

float Face::getHeight(LengthUnit lU) const {
	return UnitConverter::convert(this->height,iLU,lU);
}

void Face::setHeight(float height,LengthUnit lU) {
	this->height = UnitConverter::convert(height,lU,iLU);
}

float Face::getDistance(LengthUnit lU) const {
	return UnitConverter::convert(this->distance,iLU,lU);
}

void Face::setDistance(float distance,LengthUnit lU) {
	this->distance = UnitConverter::convert(distance,lU,iLU);
}

float Face::getAngleVariance(AngleUnit aU) const {
	return UnitConverter::convert(angleVariance,iAU,aU);
}

void Face::setAngleVariance(float angleVariance,AngleUnit aU) {
	this->angleVariance = UnitConverter::convert(angleVariance,aU,iAU);
}

float Face::getHeightVariance(LengthUnit lU) const {
	return UnitConverter::convert(heightVariance,iLU,lU);
}

void Face::setHeightVariance(float heightVariance,LengthUnit lU) {
	this->heightVariance = UnitConverter::convert(heightVariance,lU,iLU);
}

float Face::getDistanceVariance(LengthUnit lU) const {
	return distanceVariance;
}

void Face::setDistanceVariance(float distanceVariance,LengthUnit lU) {
	this->distanceVariance = distanceVariance;
}

float Face::getReliability() const {
	return reliability;
}

void Face::setReliability(float reliability) {
	this->reliability = reliability;
}

float Face::getFaceWidthAngle(AngleUnit aU) const {
	return faceWidthAngle;
}

void Face::setFaceWidthAngle(float faceWidthAngle,AngleUnit aU) {
	this->faceWidthAngle = faceWidthAngle;
}

int Face::getLineOfSight() const {
	return lineOfSight;
}

void Face::setLineOfSight(int lineOfSight) {
	this->lineOfSight = lineOfSight;
}

unsigned int Face::getImagePositionX() const {
	return imagePositionX;
}

void Face::setImagePositionX(unsigned int x) {
	this->imagePositionX = x;
}

unsigned int Face::getImagePositionY() const {
	return imagePositionY;
}

void Face::setImagePositionY(unsigned int y) {
	this->imagePositionY = y;
}

unsigned int Face::getImageWidth() const {
	return imageWidth;
}

void Face::setImageWidth(unsigned int imageWidth) {
	this->imageWidth = imageWidth;
}

unsigned int Face::getImageHeight() const {
	return imageHeight;
}

void Face::setImageHeight(unsigned int imageHeight) {
	this->imageHeight = imageHeight;
}

bool Face::operator==(const Face &other) const {

	return Type::operator==(other)
			&& abs(other.angle - angle) < TOLERANCE
			&& abs(other.distance - distance) < TOLERANCE
			&& abs(other.height - height) < TOLERANCE
			&& abs(other.angleVariance - angleVariance) < TOLERANCE
			&& abs(other.distanceVariance - distanceVariance) < TOLERANCE
			&& abs(other.heightVariance - heightVariance) < TOLERANCE
			&& abs(other.reliability - reliability) < TOLERANCE
			&& abs(other.faceWidthAngle - faceWidthAngle) < TOLERANCE
			&& (other.lineOfSight == lineOfSight)
			&& (other.imagePositionX == imagePositionX)
			&& (other.imagePositionY == imagePositionY)
			&& (other.imageWidth == imageWidth)
			&& (other.imageHeight == imageHeight);

}

ostream& operator<<(ostream& o, Face const& face) {

	return o << "Face [gen = " << face.getGenerator() << " timestamp = "
			<< face.getTimestamp() << " angle = " << face.getAngle(RADIAN)
			<< ", angleVariance = " << face.getAngleVariance(RADIAN)
			<< ", distance = " << face.getDistance(MILLIMETER) << ", distanceVariance = "
			<< face.getDistanceVariance(MILLIMETER) << ", height = " << face.getHeight(MILLIMETER)
			<< ", heightVariance = " << face.getHeightVariance(MILLIMETER)
			<< ", lineOfSight = " << face.getLineOfSight()
			<< ", faceWidthAngle = " << face.getFaceWidthAngle(RADIAN)
			<< ", reliability = " << face.getReliability()
			<< ", imagePositionX = " << face.getImagePositionX()
			<< ", imagePositionY = " << face.getImagePositionY()
			<< ", imageWdith = " << face.getImageWidth() << ", imageHeight = "
			<< face.getImageHeight() << "]";

}

void Face::sanitizeLocation(xmltio::Location &loc) const {

}

void Face::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("POSITION/@angle")] = getAngle(iAU);
	loc[XPath("POSITION/@angleVariance")] = getAngleVariance(iAU);
	loc[XPath("POSITION/@height")] = getHeight(iLU);
	loc[XPath("POSITION/@heightVariance")] = getHeightVariance(iLU);
	loc[XPath("POSITION/@distance")] = getDistance(iLU);
	loc[XPath("POSITION/@distanceVariance")] = getDistanceVariance(iLU);

	loc[XPath("IMAGEPOSITION/@x")] = getImagePositionX();
	loc[XPath("IMAGEPOSITION/@y")] = getImagePositionY();
	loc[XPath("IMAGEPOSITION/@width")] = getImageWidth();
	loc[XPath("IMAGEPOSITION/@height")] = getImageHeight();

	loc[XPath("DATA/@lineOfSight")] = getLineOfSight();
	loc[XPath("DATA/@faceWidthAngle")] = getFaceWidthAngle(iAU);
	loc[XPath("DATA/@reliability")] = getReliability();

}

void Face::fromLocation(const xmltio::Location &loc_, Face &face) {

	Type::fromLocation(loc_, face);

	face.setAngle(extract<float>(loc_[XPath("POSITION/@angle")]), iAU);
	face.setAngleVariance(
			extract<float>(loc_[XPath("POSITION/@angleVariance")]),iAU);
	face.setHeight(extract<float>(loc_[XPath("POSITION/@height")]),iLU);
	face.setHeightVariance(
			extract<float>(loc_[XPath("POSITION/@heightVariance")]),iLU);
	face.setDistance(extract<float>(loc_[XPath("POSITION/@distance")]),iLU);
	face.setDistanceVariance(
			extract<float>(loc_[XPath("POSITION/@distanceVariance")]),iLU);

	face.setImagePositionX(
			extract<unsigned int>(loc_[XPath("IMAGEPOSITION/@x")]));
	face.setImagePositionY(
			extract<unsigned int>(loc_[XPath("IMAGEPOSITION/@y")]));
	face.setImageWidth(
			extract<unsigned int>(loc_[XPath("IMAGEPOSITION/@width")]));
	face.setImageHeight(
			extract<unsigned int>(loc_[XPath("IMAGEPOSITION/@height")]));

	face.setLineOfSight(extract<int>(loc_[XPath("DATA/@lineOfSight")]));
	face.setFaceWidthAngle(extract<float>(loc_[XPath("DATA/@faceWidthAngle")]),iAU);
	face.setReliability(extract<float>(loc_[XPath("DATA/@reliability")]));

}

}
