/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * Legs.cpp
 *
 *  Created on: Aug 18, 2009
 *      Author: jwienke
 */

#include "Legs.h"

#include <iostream>

using namespace std;
using namespace xmltio;
using namespace airobots;

Legs::Legs() {
}

Legs::~Legs() {
}

std::string Legs::getBaseTag() {
	return "LEGS";
}

std::string Legs::getTemplate() {
	return string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate()
			+ "<POSITION angle=\"\" angleVariance=\"\" distance=\"\" distanceVariance=\"\" />"
			+ "<SHARING nrInvolved=\"\" involved1=\"\" involved2=\"\" isPair=\"\"/>"
			+ "</LEGS>";
}

std::string Legs::getPerceptName() {
	return "legs";
}

float Legs::getAngle(AngleUnit angleUnit) const {
	return UnitConverter::convert(this->angle,RADIAN,angleUnit);
}

void Legs::setAngle(float angle,AngleUnit angleUnit) {
	this->angle = UnitConverter::convert(angle,angleUnit,RADIAN);
}

float Legs::getDistance(LengthUnit lengthUnit) const {
	return UnitConverter::convert(this->distance,MILLIMETER,lengthUnit);
}

void Legs::setDistance(float distance,LengthUnit lengthUnit) {
	this->distance = UnitConverter::convert(distance,lengthUnit,MILLIMETER);
}

float Legs::getAngleVariance(AngleUnit angleUnit) const {
	return UnitConverter::convert(this->angleVariance,RADIAN,angleUnit);
}

void Legs::setAngleVariance(float angleVariance, AngleUnit angleUnit) {
	this->angleVariance = UnitConverter::convert(angleVariance,angleUnit,RADIAN);
}

float Legs::getDistanceVariance(LengthUnit lengthUnit) const {
	return UnitConverter::convert(this->distanceVariance,MILLIMETER,lengthUnit);
}

void Legs::setDistanceVariance(float distanceVariance, LengthUnit lengthUnit) {
	this->distanceVariance = UnitConverter::convert(distanceVariance,lengthUnit,MILLIMETER);
}

int Legs::getNumInvovled() const {
	return numInvolved;
}

void Legs::setNumInvolved(int numInvovled) {
	this->numInvolved = numInvovled;
}

int Legs::getInvolved1() const {
	return involved[0];
}

void Legs::setInvolved1(int involved) {
	this->involved[0] = involved;
}

int Legs::getInvolved2() const {
	return involved[1];
}

void Legs::setInvolved2(int involved) {
	this->involved[1] = involved;
}

bool Legs::isPair() const {
	return pair;
}

void Legs::setPair(bool isPair) {
	this->pair = isPair;
}

bool Legs::operator==(const Legs &other) const {
	return Type::operator==(other) && (other.angle == angle) && (other.distance
			== distance) && (other.angleVariance == angleVariance)
			&& (other.distanceVariance == distanceVariance)
			&& (other.involved[0] == involved[0]) && (other.involved[1]
			== involved[1]) && (other.numInvolved == numInvolved)
			&& (other.pair == pair);
}

std::ostream& airobots::operator<<(std::ostream& o, Legs const& legs) {
	return o << "Legs [angle = " << legs.getAngle(RADIAN) << ", angleVariance = "
			<< legs.getAngleVariance(RADIAN) << ", distance = " << legs.getDistance(MILLIMETER)
			<< ", distanceVariance = " << legs.getDistanceVariance(MILLIMETER)
			<< ", numInvolved = " << legs.getNumInvovled() << ", involdved1 = "
			<< legs.getInvolved1() << ", involved2 = " << legs.getInvolved2()
			<< ", isPair = " << legs.isPair() << "]";
}

void Legs::sanitizeLocation(xmltio::Location &loc) const {
	
}

void Legs::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("POSITION/@angle")] = getAngle(RADIAN);
	loc[XPath("POSITION/@angleVariance")] = getAngleVariance(RADIAN);
	loc[XPath("POSITION/@distance")] = getDistance(MILLIMETER);
	loc[XPath("POSITION/@distanceVariance")] = getDistanceVariance(MILLIMETER);

	loc[XPath("SHARING/@nrInvolved")] = getNumInvovled();
	loc[XPath("SHARING/@involved1")] = getInvolved1();
	loc[XPath("SHARING/@involved2")] = getInvolved2();
	loc[XPath("SHARING/@isPair")] = isPair();

}

void Legs::fromLocation(const xmltio::Location &loc_, Legs &legs) {

	Type::fromLocation(loc_, legs);

	legs.setAngle(extract<float> (loc_[XPath("POSITION/@angle")]),RADIAN);
	legs.setAngleVariance(extract<float> (
			loc_[XPath("POSITION/@angleVariance")]),RADIAN);
	legs.setDistance(extract<float> (loc_[XPath("POSITION/@distance")]),MILLIMETER);
	legs.setDistanceVariance(extract<float> (loc_[XPath(
			"POSITION/@distanceVariance")]),MILLIMETER);

	legs.setNumInvolved(extract<int> (loc_[XPath("SHARING/@nrInvolved")]));
	legs.setInvolved1(extract<int> (loc_[XPath("SHARING/@involved1")]));
	legs.setInvolved2(extract<int> (loc_[XPath("SHARING/@involved2")]));
	legs.setPair(extract<int> (loc_[XPath("SHARING/@isPair")]));

}
