/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.navigation;

import nu.xom.Element;
import nu.xom.Nodes;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.common.ElementParser;

/**
 * Domain class representing the result of a goalReached query.
 *
 * @author lschilli
 * @author jwienke
 */
public class GoalReachedData extends Type {

	private boolean stopped;

        /**
         * Indicates the reach state.
         * <b>Warning:</b> <code>true</code> does not mean, that the robot is
         * stopped. It has only reached its goal within the given tolerance
         * values.
         * @return true if the goal was reached, false otherwise
         */
	public boolean isStopped() {
		return stopped;
	}

	public void setStopped(boolean stopped) {
		this.stopped = stopped;
	}

	public static String getBaseTag() {
		return "GOALREACHED";
	}

	public static void fromElement(Element element, GoalReachedData type) {

		Type.fromElement(element, type);

		Nodes nodes = element.query("REACHED");
		if (nodes.size() > 0) {
			Element elem = (Element) nodes.get(0);
			type.setStopped("1".equals(ElementParser.getTagValue(elem)));
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element reachedElement = new Element("REACHED");
		parent.appendChild(reachedElement);

		String reachedString = "0";
		if (isStopped()) {
			reachedString = "1";
		}
		reachedElement.appendChild(reachedString);

	}

}
