package de.unibi.airobots.btl.data.taskpattern;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.data.common.ElementParser;

/**
 * is send by the system to the dialog, for a self following suggestion.
 * the dialog then initiates the follow task if needed.
 * @author arothert
 */
public class SelfFollowingSuggestion extends TaskPatternBase{
	
	
	public SelfFollowingSuggestion(){
		
	}

	
	/**
	 * the base tag should be named after the class itself.
	 * @return
	 */
	public static String getBaseTag() {
		return "SELF_FOLLOWING_SUGGESTION";
	}
	

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);
		Element statesElement = 
			TaskPatternStatusTagData.createTaskStatusElement(getOrigin(), getValue());
		parent.appendChild(statesElement);
	}
	
	public static void fromElement(Element element, SelfFollowingSuggestion type) {
		
		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				
				if (elem.getLocalName().equals(STATUS_TAG)) {
					type.setValue(ElementParser.getAttributeValue(elem, VALUE_ATT));
					type.setOrigin(ElementParser.getAttributeValue(elem, ORIGIN_ATT));			
				} 
			}
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("["+getBaseTag()+": ");
		strb.append("[STATUS: ");
		strb.append("value: " + getValue() + "; ");
		strb.append("origin: " + getOrigin() + "]] ");
		return strb.toString();
	}

}
