/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/
package de.unibi.airobots.btl.data.geometry;

import static org.junit.Assert.*;
import nu.xom.Element;

import org.junit.Test;

import de.unibi.airobots.btl.data.geometry.PrecisePolygon;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * Test cases for {@link Type}.
 * 
 * @author lziegler
 */
public class PrecisePolygonTest {

	@Test
	public void selfCompatibility() throws Exception {

		PrecisePolygon p0 = new PrecisePolygon();
		p0.addPoint(0, 0, LengthUnit.MILLIMETER);
		p0.addPoint(2, 0, LengthUnit.MILLIMETER);
		p0.addPoint(0, 2, LengthUnit.MILLIMETER);

		PrecisePolygon p1 = new PrecisePolygon();
		p1.addPoint(0, 0, LengthUnit.MILLIMETER);
		p1.addPoint(2, 0, LengthUnit.MILLIMETER);

		assertFalse("equals() not working", p0.equals(p1));
		assertFalse("equals() not working", p1.equals(p0));

		p1.addPoint(0, 2, LengthUnit.MILLIMETER);

		assertTrue("equals() not working", p0.equals(p1));
		assertTrue("equals() not working", p1.equals(p0));

		p1.addPoint(0, 3, LengthUnit.MILLIMETER);

		assertFalse("equals() not working", p0.equals(p1));
		assertFalse("equals() not working", p1.equals(p0));

		Element element = new Element(PrecisePolygon.getBaseTag());
		p0.fillInto(element);
		PrecisePolygon p2 = PrecisePolygon.fromElement(element);

		assertTrue("serializing not working", p0.equals(p2));
	}
}
