/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/
package de.unibi.airobots.btl.data.person;

import static org.junit.Assert.*;

import java.util.ArrayList;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import java.util.Map;
import java.util.HashMap;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.Type;
import de.unibi.airobots.btl.data.person.AnchorAttribute;

/**
 * Test cases for {@link Type}.
 * 
 * @author dschulze <dschulze@techfak.uni-bielefeld.de>
 */
public class AnchorAttributeTest {

	@Test
	public void selfCompatibility() {
	
		AnchorAttribute original = new AnchorAttribute("size");

		original.addValue("width", 10d);
		original.addValue("height", 20d);

		Document doc = new Document(new Element(AnchorAttribute.getBaseTag()));
		original.fillInto(doc.getRootElement());

		System.out.println(doc.toXML());

		AnchorAttribute parsed = new AnchorAttribute();
		AnchorAttribute.fromElement(doc.getRootElement(), parsed);

		HashMap<String, Double> map1 = new HashMap<String, Double>();
		HashMap<String, Double> map2 = new HashMap<String, Double>();

		map1.put("Name", 12.3); map1.put("Real", 15.6);
		map2.put("Name", 12.3); map2.put("Real", 15.6);

		map1.equals(map2);

		assertEquals(original.getName(), parsed.getName());
		assertEquals(original.getValues(), parsed.getValues());

	}

	@Test
	public void fusionTest() {
		
		AnchorAttribute attr1 = new AnchorAttribute("size");
		AnchorAttribute attr2 = new AnchorAttribute("size");

		attr1.addValue("width", 10d); attr1.addValue("height", 20d);
		attr2.addValue("width", 10.5); attr2.addValue("height", 20.5d);

		Map<String, Double> values1 = attr1.getValues();
		Map<String, Double> values2 = attr2.getValues();
		
		assertEquals(10d, values1.get("width"), 0.0001);
		assertEquals(20d, values1.get("height"), 0.0001);
		assertEquals(10.5, values2.get("width"), 0.0001);
		assertEquals(20.5, values2.get("height"), 0.0001);

		attr1.fuseMean(attr2);

		values1 = attr1.getValues();
		values2 = attr2.getValues();
		
		assertEquals(10.25, values1.get("width"), 0.0001);
		assertEquals(20.25, values1.get("height"), 0.0001);
		assertEquals(10.5, values2.get("width"), 0.0001);
		assertEquals(20.5, values2.get("height"), 0.0001);


	}

}
